@@ -262,6 +262,8 @@ public:
 		void setFraming(timeout_t frame);
 	};
 
+	static Level tolevel(float dbm);
+	static float todbm(Level l);
 	static bool hasDevice(unsigned index = 0);
 	static AudioDevice *getDevice(unsigned index = 0, DeviceMode = PLAY);
 	static const char *getCodecPath(void);
@@ -280,8 +282,6 @@ public:
 	static bool isEndian(Info &info);
 	static bool swapEndian(Encoding encoding, void *buffer, unsigned samples);
 	static bool swapEndian(Info &info, void *buffer, unsigned samples);
-	static Linear getSingleTone(Linear buffer, timeout_t timeout, unsigned f1, Level m1 = 32767, Rate rate = rate8khz);
-	static Linear getDualTone(Linear buffer, timeout_t timeout, unsigned f1, unsigned f2, Level m1 = 32767, Level m2 = 32767, Rate rate = rate8khz);
 	static Level getImpulse(Encoding encoding, void *buffer, unsigned samples);
 	static Level getImpulse(Info &info, void *buffer, unsigned samples = 0);
         static Level getPeak(Encoding encoding, void *buffer, unsigned samples);
@@ -342,12 +342,40 @@ public:
  */
 class __EXPORT AudioTone : public Audio
 {
-private:
+protected:
 	Rate rate;
 	unsigned samples;
 	Linear frame;
 	double df1, df2, p1, p2;
 	Level m1, m2;
+	bool silencer;
+
+	/**
+	 * Set the frame to silent.
+	 */	
+	void silence(void);
+
+	void reset(void);
+
+	void cleanup(void);
+
+	/**
+	 * Set frame to generate single tone...
+	 *
+	 * @param freq of tone.
+	 * @param audio level of tone.
+	 */
+	void single(unsigned freq, Level l);
+
+	/**
+	 * Set frame to generate dual tone...
+	 *
+	 * @param freq of tone 1
+	 * @param freq of tone 2
+	 * @param level of tone 1
+	 * @param level of tone 2
+	 */
+	void dual(unsigned freq1, unsigned freq2, Level l1, Level l2);
 
 public:
 	inline Rate getRate(void)
@@ -356,6 +384,8 @@ public:
 	inline size_t getSamples(void)
 		{return samples;};
 
+	bool isSilent(void);
+
 	/**
 	 * Iterate the tone frame, and extract linear samples in
 	 * native frame.  If endian flag passed, then convert for
@@ -363,7 +393,33 @@ public:
 	 *
 	 * @return pointer to samples.
 	 */
-	Linear getFrame(bool endian = false);	
+	virtual Linear getFrame(bool endian = false);	
+
+	/**
+	 * This is used to copy one or more pages of framed audio
+	 * quickly to an external buffer.
+	 *
+	 * @return number of frames copied.
+	 * @param buffer to copy into.
+	 * @param number of frames requested.
+	 */
+	unsigned getFrames(Linear buffer, unsigned count);
+
+	/**
+	 * See if at end of tone.  This is used for non-continues audio
+	 * tones, or to detect "break" events.
+	 *
+	 * @return true if end of data.
+	 */
+	virtual bool isComplete(void);
+
+	/**
+	 * Construct a silent tone generator of specific frame size.
+	 *
+	 * @param frame duration in milliseconds.
+	 * @param sample rate.
+	 */
+	AudioTone(timeout_t duration = 20, Rate r = rate8khz);
 
 	/**
 	 * Construct a dual tone frame generator.
@@ -387,7 +443,7 @@ public:
 	 */
 	AudioTone(unsigned freq, Level l, timeout_t duration = 20, Rate r = rate8khz); 
 
-	~AudioTone();
+	virtual ~AudioTone();
 };
 
 /**
@@ -730,8 +786,7 @@ public:
 	AudioStream(const char *filename, Mode m = modeRead, timeout_t framing = 0);
 	AudioStream(const char *filename, Info *info, bool excl = false, timeout_t framing = 0);
 
-	virtual ~AudioStream()
-		{close();};
+	virtual ~AudioStream();
 
 	void open(const char *filename, Mode m = modeRead, timeout_t framing = 0);
 	void create(const char *fname, Info *info, bool exclusive = false, timeout_t framing = 0);
@@ -911,31 +966,140 @@ public:
 		{return enabled;};
 };
 
+
+class __EXPORT TelTone : public AudioTone
+{
+public:
+	typedef struct _tonedef       
+        {
+                struct _tonedef *next;
+                timeout_t duration, silence;
+                unsigned count;
+                unsigned short f1, f2;
+        } tonedef_t;
+
+        typedef struct _tonekey
+        {
+                struct _tonekey *next;
+                struct _tonedef *first;
+                struct _tonedef *last;
+                char id[1];
+        } tonekey_t;
+
+	TelTone(tonekey_t *k, Level l, timeout_t frame = 20);
+	~TelTone();
+
+	Linear getFrame(bool endian = false);
+	bool isComplete(void);
+
+
+	/**
+	 * Load a teltones database file into memory.
+	 *
+	 * @return true if successful
+	 * @param pathname of file to load.
+	 * @param optionally load only one locale
+	 */
+	static bool load(const char *pathname, const char *locale = NULL);
+
+	/**
+	 * find an entry in the teltones database.
+	 *
+	 * @return key of tone list if found, else NULL
+	 * @param name of tone
+	 * @param optional locale to search under
+	 */
+	static tonekey_t *find(const char *tone, const char *locale = NULL);
+
+protected:
+	tonekey_t *tone;
+	tonedef_t *def;
+	unsigned remaining, silent, count;
+	timeout_t framing;
+	Level level;	
+	bool complete;
+};
+
+/**
+ * DTMFTones is used to generate a series of dtmf audio data from a
+ * "telephone" number passed as an ASCII string.  Each time getFrame()
+ * is called, the next audio frame of dtmf audio will be created 
+ * and pulled.  
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short Generate DTMF audio
+ */
+class __EXPORT DTMFTones : public AudioTone
+{
+protected:
+	unsigned remaining, dtmfframes;
+	timeout_t frametime;
+	const char *digits;
+	Level level;
+	bool complete;
+
+public:
+	DTMFTones(const char *digits, Level l, timeout_t duration = 20, timeout_t interdigit = 60);
+
+	~DTMFTones();
+
+	Linear getFrame(bool endian = false);
+	bool isComplete(void);
+};
+
+/**
+ * MFTones is used to generate a series of mf audio data from a
+ * "telephone" number passed as an ASCII string.  Each time getFrame()
+ * is called, the next audio frame of dtmf audio will be created 
+ * and pulled.  
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short Generate MF audio
+ */
+class __EXPORT MFTones : public AudioTone
+{
+protected:
+	unsigned remaining, mfframes;
+	timeout_t frametime;
+	const char *digits;
+	Level level;
+	bool complete, kflag;
+
+public:
+	MFTones(const char *digits, Level l, timeout_t duration = 20, timeout_t interdigit = 60);
+
+	~MFTones();
+
+	Linear getFrame(bool endian = false);
+	bool isComplete(void);
+};
+
+
 /**
  * DTMFDetect is used for detecting DTMF tones in a stream of audio.
  * It currently only supports 8000Hz input.
  */
-
 class __EXPORT DTMFDetect : public Audio
 {
 public:
-	DTMFDetect();
-	~DTMFDetect();
+        DTMFDetect();
+        ~DTMFDetect();
 
-	int putSamples(Linear buffer, int count);
-	int getResult(char *buf, int max);
+        int putSamples(Linear buffer, int count);
+        int getResult(char *buf, int max);
 protected:
-	void goertzelInit(goertzel_state_t *s, tone_detection_descriptor_t *t);
-	void goertzelUpdate(goertzel_state_t *s, Sample x[], int samples);
-	float goertzelResult(goertzel_state_t *s);
+        void goertzelInit(goertzel_state_t *s, tone_detection_descriptor_t *t);
+        void goertzelUpdate(goertzel_state_t *s, Sample x[], int samples);
+        float goertzelResult(goertzel_state_t *s);
+
 private:
-	dtmf_detect_state_t *state;
-	tone_detection_descriptor_t dtmf_detect_row[4];
-	tone_detection_descriptor_t dtmf_detect_col[4];
-	tone_detection_descriptor_t dtmf_detect_row_2nd[4];
-	tone_detection_descriptor_t dtmf_detect_col_2nd[4];
-	tone_detection_descriptor_t fax_detect;
-	tone_detection_descriptor_t fax_detect_2nd;
+        dtmf_detect_state_t *state;
+        tone_detection_descriptor_t dtmf_detect_row[4];
+        tone_detection_descriptor_t dtmf_detect_col[4];
+        tone_detection_descriptor_t dtmf_detect_row_2nd[4];
+        tone_detection_descriptor_t dtmf_detect_col_2nd[4];
+        tone_detection_descriptor_t fax_detect;
+        tone_detection_descriptor_t fax_detect_2nd;
 };
 
 }
