@@ -87,10 +87,8 @@ AudioCodec *AudioFile::getCodec(void)
 	}
 }
 
-void AudioFile::setShort(unsigned char *data, unsigned short sval)
+void AudioFile::setShort(unsigned char *data, unsigned short val)
 {
-	snd16_t val = (snd16_t)sval;
-
 	if(info.order == __BIG_ENDIAN)
 	{
 		data[0] = val / 256;
@@ -111,10 +109,9 @@ unsigned short AudioFile::getShort(unsigned char *data)
 		return data[1] * 256 + data[0];
 }
 
-void AudioFile::setLong(unsigned char *data, unsigned long lval)
+void AudioFile::setLong(unsigned char *data, unsigned long val)
 {
 	int i = 4;
-	snd32_t val = (snd32_t)lval;
 
 	while(i-- > 0)
 	{
@@ -166,6 +163,7 @@ AudioFile::AudioFile(const char *name, Info *info, unsigned long samples)
 AudioFile::~AudioFile()
 {
 	AudioFile::close();
+	AudioFile::clear();
 }
 
 void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t framing)
@@ -935,11 +933,13 @@ void AudioFile::close(void)
 
 	if(! afSeek(0)) {
 		afClose();
+		clear();
 		return;
 	}	
 
 	if(-1 == afRead(buf, 58)) {
 		afClose();
+		clear();
 		return;
 	}
 
@@ -958,12 +958,11 @@ void AudioFile::close(void)
 		else
 			setLong(buf + 40, length - header); 
 
-		if(afWrite(buf, 58) != 58)
-			return;
-
+		afWrite(buf, 58);
 		break;
 	case snd:
-		// FIXME not implemented
+		setLong(buf + 8, length - header);
+		afWrite(buf, 12);
 		break;
 	case raw:
 		break;
