@@ -70,12 +70,15 @@ extern "C" {
 #define M_PI    3.14159265358979323846
 #endif
 
-class AudioRegistry
+using namespace ost;
+
+class AudioRegistry 
 {
 public:
 	AudioRegistry();
 
 	const char *codecdir;
+
 };
 
 #ifdef	W32
@@ -126,13 +129,22 @@ AudioRegistry::AudioRegistry()
 #endif	
 }
 
-using namespace ost;
-
 const char *Audio::getCodecPath(void)
 {
 	return registry.codecdir;
 }
 
+Audio::Level Audio::tolevel(float dbm)
+{
+	double l = pow(10.0, (dbm - M_PI)/20.0)*(32768.0*0.70711);
+	return (Level)(l*l);
+}
+
+float Audio::todbm(Level l)
+{
+	return (float)(log10(sqrt((float)l)/(32768.0*0.70711))*20.0 + M_PI);
+}
+
 Audio::Encoding Audio::getMono(Encoding encoding)
 {
 	switch(encoding)
@@ -307,54 +319,6 @@ bool Audio::isLinear(Encoding encoding)
 	return false;
 }
 
-Audio::Linear Audio::getDualTone(Linear buffer, timeout_t duration, unsigned f1, unsigned f2, Level m1, Level m2,Rate rate)
-{
-        double df1 = (f1 * M_PI * 2) / (long)rate;
-	double df2 = (f2 * M_PI * 2) / (long)rate;
-        double p1 = 0, p2 = 0;
-	size_t count;
-        size_t samples = count = (duration * (long)rate) / 1000;
-        Linear data;
-	m1 /= 2;
-	m2 /= 2;
-
-        if(!buffer)
-                buffer = new Sample[samples];
-
-        data = buffer;
-        while(count--)
-        {
-                *(data++) = (Level)(sin(p1) * (double)m1) + 
-			(Level)(sin(p2) * (double)m2);
-                p1 += df1;
-		p2 += df2;
-        }
-
-        return data;
-}
-
-Audio::Linear Audio::getSingleTone(Linear buffer, timeout_t duration, unsigned f1, Level m1, Rate rate)
-{
-	double freq = (f1 * M_PI * 2) / (long)rate;
-	double pos = 0;
-	size_t count;
-	size_t samples = count = (duration * (long)rate) / 1000;
-	Linear data;
-
-	if(!buffer)
-		buffer = new Sample[samples];
-
-	data = buffer;
-	while(count--)
-	{
-		*(data++) = (Level)(sin(pos) * (double)m1);
-		pos += freq;
-	}
-
-	return data;
-}
-
-
 Audio::Level Audio::getImpulse(Encoding encoding, void *buffer, unsigned samples)
 {
 	unsigned long sum = 0;
