@@ -123,6 +123,12 @@ AudioStream::AudioStream(const char *fname, Info *info, bool exclusive, timeout_
 	create(fname, info, exclusive, framing);
 }
 
+AudioStream::~AudioStream()
+{
+	AudioStream::close();
+	AudioFile::clear();
+}
+
 bool AudioStream::isStreamable(void)
 {
 	if(!isOpen())
@@ -168,7 +174,7 @@ void AudioStream::close(void)
 
 	framebuf = NULL;
 	codec = NULL;
-	AudioFile::clear();
+	AudioFile::close();
 }
 
 void AudioStream::create(const char *fname, Info *info, bool exclusive, timeout_t framing)
